unit opencv;

interface

  const
    CV_INTER_NN = 0;
    CV_INTER_LINEAR = 1;
    CV_INTER_CUBIC = 2;
    CV_INTER_AREA = 3;
    CV_INTER_LANCZOS4 = 4;


    CV_BGR2GRAY        = 6;
  type

    pCvSize = ^TCvSize;

    TCvSize = packed record
      width: Integer;
      height: Integer;
    end;

    TCVChar = AnsiChar;
    pCVChar = pAnsiChar;
    CVChar = AnsiChar;

    TCvCapture = packed record
    end;

    pCvRect = ^TCvRect;

    TCvRect = packed record
      x: Integer;
      y: Integer;
      width: Integer;
      height: Integer;
    end;

    pCvPoint = ^TCvPoint;

    TCvPoint = packed record
      x: Integer;
      y: Integer;
    end;

    pCvScalar = ^TCvScalar;

    TCvScalar = packed record
      val: array [0 .. 3] of Double;
    end;



    pCvFont = ^TCvFont;

    TCvFont = packed record
      nameFont: pCvChar;
      color: TCvScalar;
      font_face: Integer;
      ascii: pInteger;
      greek: pInteger;
      cyrillic: pInteger;
      hscale, vscale: Single;
      shear: Single;
      thickness: Integer;
      dx: Single;
      line_type: Integer;
    end;


    pCvCapture = ^TCvCapture;

    pIplImage = ^TIplImage;
    pIplROI = ^TIplROI;
    pIplTileInfo = ^TIplTileInfo;
    TA4CVChar = array [0 .. 3] of CVChar;

    TiplCallBack = procedure(img: pIplImage; xIndex: Integer; yIndex: Integer; mode: Integer);

    TIplTileInfo = packed record
      callBack: TiplCallBack;
      id: Pointer;
      tileData: pCVChar;
      width: Integer;
      height: Integer;
    end;

    TIplROI = packed record
      coi: Integer; 
      xOffset: Integer;
      yOffset: Integer;
      width: Integer;
      height: Integer;
    end;

    TIplImage = packed record
      nSize: Integer; (*sizeof(IplImage)*)
      id: Integer; (*version (=0)*)
      nChannels: Integer; (*Most of OpenCV functions support 1,2,3 or 4 channels*)
      alphaChannel: Integer; (*Ignored by OpenCV*)
      depth: Integer; (*Pixel depth in bits: Pixel depth in bits: IPL_DEPTH_8U, IPL_DEPTH_8S, IPL_DEPTH_16S,
        IPL_DEPTH_32S, IPL_DEPTH_32F and IPL_DEPTH_64F are supported.*)
      colorModel: TA4CVChar; (*Ignored by OpenCV*)
      channelSeq: TA4CVChar; (*ditto*)
      dataOrder: Integer; (*0 - interleaved color channels, 1 - separate color channels.*)
      origin: Integer; (*0 - top-left origin,*)
      align: Integer; (*Alignment of image rows (4 or 8).*)
      width: Integer; (*Image width in pixels.*)
      height: Integer; (*Image height in pixels.*)
      roi: pIplROI; (*Image ROI. If NULL, the whole image is selected.*)
      maskROI: pIplImage; (*Must be NULL.*)
      imageId: Pointer; (*"           "*)
      tileInfo: pIplTileInfo; (*"           "*)
      imageSize: Integer; (*Image data size in bytes*)
      imageData: pByte; (*Pointer to aligned image data.*)
      widthStep: Integer; (*Size of aligned image row in bytes.*)
      BorderMode: array [0 .. 3] of Integer; (*Ignored by OpenCV.*)
      BorderConst: array [0 .. 3] of Integer; (*Ditto.*)
      imageDataOrigin: pByte; (*Pointer to very origin of image data*)
    end;

  pCvArr = Pointer;

  function cvCreateImage(size: TCvSize; depth, channels: Integer): pIplImage; cdecl;
  procedure cvReleaseImage(var image: pIplImage); cdecl;
  function cvGetSize(const arr: pCvArr): TCvSize;
  procedure _cvGetSize(const arr: pCvArr; Var size: TCvSize); cdecl;
  procedure cvZero(arr: pCvArr); cdecl;
  procedure cvRectangle(img: pCvArr; pt1: TCvPoint; pt2: TCvPoint; color: TCvScalar; thickness: Integer = 1; line_type: Integer = 8; shift: Integer = 0); cdecl;
  procedure cvInitFont(font: pCvFont; font_face: Integer; hscale: double; vscale: double; shear: double = 0; thickness: Integer = 1; line_type: Integer = 8); cdecl;
  procedure cvPutText(img: pCvArr; const text: pCvChar; org: TCvPoint; const font: pCvFont; color: TCvScalar); cdecl;

  procedure cvResize(const src: pIplImage; dst: pIplImage; interpolation: Integer = CV_INTER_LINEAR); cdecl;
  procedure cvCvtColor(const src: pIplImage; dst: pIplImage; code: Integer); cdecl;


  function cvQueryFrame(capture: pCvCapture): pIplImage; cdecl;
  function cvCreateFileCapture(const filename: pCVChar): pCvCapture; cdecl;
  function cvCreateCameraCapture(index: Longint): pCvCapture; cdecl;
  procedure cvReleaseCapture(Var capture: pCvCapture); cdecl;


  function CV_RGB(const r, g, B: double): TCvScalar;
  function CV_POINT(const x, y : integer): TCvPoint;

implementation

  function cvCreateImage; external 'opencv_core246.dll';
  procedure cvReleaseImage; external 'opencv_core246.dll';
  procedure _cvGetSize; external 'opencv_core246.dll' name 'cvGetSize';
  procedure cvZero; external 'opencv_core246.dll' name 'cvSetZero';
  procedure cvRectangle; external 'opencv_core246.dll';
  procedure cvInitFont; external 'opencv_core246.dll';
  procedure cvPutText; external 'opencv_core246.dll';

function cvGetSize(const arr: pCvArr): TCvSize; assembler;
asm
  //   OpenCV cvGetSize    eax:edx        arr,
  //     ,          arr,
  //     (  ),
  //     , ,    arr  result
  //            

  push edx
  push eax
  call _cvGetSize
  pop ecx
  mov ecx,edx
  pop edx
  mov @Result.height,ecx
  mov @Result.width,eax
end;


  procedure cvResize; external 'opencv_imgproc246';
  procedure cvCvtColor(const src: pIplImage; dst: pIplImage; code: Integer); external 'opencv_imgproc246.dll' name 'cvCvtColor';

  function cvQueryFrame; external 'opencv_highgui246.dll';
  function cvCreateFileCapture; external 'opencv_highgui246.dll';
  function cvCreateCameraCapture; external 'opencv_highgui246.dll';
  procedure cvReleaseCapture; external 'opencv_highgui246.dll';


  function CvScalar(const val0: Double; const val1: Double = 0; const val2: Double = 0; const val3: Double = 0): TCvScalar;
  begin
    Result.val[0] := val0;
    Result.val[1] := val1;
    Result.val[2] := val2;
    Result.val[3] := val3;
  end;

  function CV_RGB(const r, g, B: double): TCvScalar;
  begin
    Result := CvScalar(B, g, r, 0);
  end;

  function CV_POINT(const x, y : integer): TCvPoint;
  begin
    Result.x := x;
    Result.y := y;
  end;

end.

